==> kdrill3.1.tar.gz: kdrill3.1/grades.c <==
/* "grades.c"
 *	has code for selecting grade levels,
 *	PLUS init for the showinorder stuffs
 */

#include <stdio.h>

#include <ctype.h>
#include <Intrinsic.h>
#include <StringDefs.h>
#include <Xaw/Command.h>
#include <Xaw/Label.h>
#include <Xaw/Form.h>
#include "defs.h"
#include "externs.h"


#define FULLWIDTH 320
Widget gradelevelForm;

/* [0] is "all" button */
Widget gradeButtons[8];
int gradelevelflags;

/* normally, I would make this inline, but
 * " grades.c:20: warning: ANSI does not permit the keyword `inline' "
 * Hurmph....
 *   So, This routine reverses black/white values of a widget
 */
void ReverseButton(Widget w){
	XtVaSetValues(w,XtNbackground,black,
		       XtNforeground,white,
		       NULL);
}
/* DO NOT make this inline ...
 * used elsewhere.
 * Undoes what ReverseButton() does
 */
void UnreverseButton(Widget w){
	XtVaSetValues(w,XtNbackground,white,
		       XtNforeground,black,
		       NULL);
}

/* printgrades:
 *	update grade widgets, cause the stupid things don't seem to want
 *	to start as they SHOULD start. Grrr...
 */

void printgrades(){
	int buttonloop;
	for(buttonloop=1;buttonloop<8;buttonloop++){
		if(gradelevelflags & (1<<buttonloop)){
			ReverseButton(gradeButtons[buttonloop]);
		} else {
			UnreverseButton(gradeButtons[buttonloop]);
		}
	}
}

/* parsegrades:
 *  initialize gradelevelflags, according to ascii string
 *	(containing single word)
 *	If string has no acceptable grade levels, we will
 *	set to ALL grades.
 */
void parsegrades(char *gradestring){
	char *parse = gradestring;
	gradelevelflags = 0;

	while(*parse && (!isspace(*parse )) ){
	    switch(*parse){
		case '1': case '2':
		case '3': case '4':
		case '5': case '6':
			gradelevelflags |= (1 << (*parse -  '0') );
			break;
		case '+':
			gradelevelflags |= ABOVESIXGRADE;
			break;
	    }
	    parse++;
	}

	if(gradelevelflags == 0)
		gradelevelflags = ALLGRADES;
}

/* GradeCalback:
 *	calback for different grade buttons
 */
void GradeCallback(Widget w,XtPointer data,XtPointer calldata){
	char statusbuff[100];
	int grademask;
	grademask = 1 << (int) data;
	
	switch((int) data){
		case 1:	case 2:
		case 3:	case 4:
		case 5: case 6:
		case 7: /* + */
			/* already on? */
			if(gradelevelflags & grademask){
				gradelevelflags -= grademask;
				if(! gradelevelflags) {
					setstatus("Must have at least one grade level!");
					Beep();
					gradelevelflags |= grademask;
					return;
				} else {
					CountKanji();
					if(numberofkanji < 9 ) {
						setstatus("Too few kanji");
						gradelevelflags |= grademask;
						Beep();
						CountKanji();
						return;
						
					}
					UnreverseButton(w);
				}
                   	} else {
			/* nope.. turn on grade level */
				ReverseButton(w);
				gradelevelflags |= grademask;
			}
			break;

		case 0:/* "All" button */
			gradelevelflags = ALLGRADES;
			printgrades();
			break;
	}
	sprintf(statusbuff,"%d characters active",numberofkanji);
	setstatus(statusbuff);
}

/* MakeGradeSelect()
 *	create buttons to select grades 1-6, plus "all" and "+"  buttons
 */
void MakeGradeSelect(){
	Widget gradelevelLabel;
	
	int widgetcount;
	gradelevelForm = XtVaCreateManagedWidget(
			"gradedisplay",formWidgetClass,buttonform,
			XtNhorizDistance,GRADEDISTANCE,
			XtNfromHoriz,usefilebutton,
			XtNvertDistance,20,
			XtNresize,False,
			NULL);

	gradelevelLabel = XtVaCreateManagedWidget(
			"gradelabel",labelWidgetClass,gradelevelForm,
			XtNlabel,"        Grade Select",XtNborderWidth,0,
			NULL);

	gradeButtons[1] = XtVaCreateWidget("grade1",commandWidgetClass,gradelevelForm,
			 XtNlabel,"1",
			 XtNfromVert,gradelevelLabel,NULL);

	gradeButtons[2] = XtVaCreateWidget("grade2",commandWidgetClass,gradelevelForm,
			 XtNlabel,"2",XtNfromHoriz,gradeButtons[1],
			 XtNfromVert,gradelevelLabel,NULL);
	gradeButtons[3] = XtVaCreateWidget("grade3",commandWidgetClass,gradelevelForm,
			 XtNlabel,"3",XtNfromHoriz,gradeButtons[2],
			 XtNfromVert,gradelevelLabel,NULL);
	gradeButtons[4] = XtVaCreateWidget("grade4",commandWidgetClass,gradelevelForm,
			 XtNlabel,"4",XtNfromHoriz,gradeButtons[3],
			 XtNfromVert,gradelevelLabel,NULL);
	gradeButtons[5] = XtVaCreateWidget("grade5",commandWidgetClass,gradelevelForm,
			 XtNlabel,"5",XtNfromHoriz,gradeButtons[4],
			 XtNfromVert,gradelevelLabel,NULL);
	gradeButtons[6] = XtVaCreateWidget("grade6",commandWidgetClass,gradelevelForm,
			 XtNlabel,"6",XtNfromHoriz,gradeButtons[5],
			 XtNfromVert,gradelevelLabel,NULL);
	gradeButtons[7] = XtVaCreateWidget("grade+",commandWidgetClass,gradelevelForm,
			 XtNlabel,"+",XtNfromHoriz,gradeButtons[6],
			 XtNfromVert,gradelevelLabel,NULL);

	gradeButtons[0] = XtVaCreateWidget("allgrades",commandWidgetClass,gradelevelForm,
			  XtNlabel," All ", XtNfromHoriz,gradeButtons[7],
			  XtNfromVert,gradelevelLabel,NULL);

	XtManageChildren(gradeButtons,8);

	for(widgetcount=0;widgetcount<8;widgetcount++){
		XtAddCallback(gradeButtons[widgetcount],XtNcallback,GradeCallback,(XtPointer) widgetcount);
	}
	/* select ALL.. */
	
}

==> kdrill3.2.tar.gz: kdrill3.2/grades.c <==
/* "grades.c"
 *	has code for selecting grade levels,
 *	PLUS init for the showinorder stuffs
 */

#include <stdio.h>

#include <ctype.h>
#include <Intrinsic.h>
#include <StringDefs.h>
#include <Xaw/Command.h>
#include <Xaw/Label.h>
#include <Xaw/Form.h>
#include "defs.h"
#include "externs.h"


Widget gradelevelForm;

/* [0] is "all" button */
Widget gradeButtons[8];
int gradelevelflags;

/* normally, I would make this inline, but
 * " grades.c:20: warning: ANSI does not permit the keyword `inline' "
 * Hurmph....
 *   So, This routine reverses black/white values of a widget
 */
void ReverseButton(Widget w){
	XtVaSetValues(w,XtNbackground,black,
		       XtNforeground,white,
		       NULL);
}
/* DO NOT make this inline ...
 * used elsewhere.
 * Undoes what ReverseButton() does
 */
void UnreverseButton(Widget w){
	XtVaSetValues(w,XtNbackground,white,
		       XtNforeground,black,
		       NULL);
}

/* printgrades:
 *	update grade widgets, cause the stupid things don't seem to want
 *	to start as they SHOULD start. Grrr...
 */

void printgrades(){
	int buttonloop;
	for(buttonloop=1;buttonloop<8;buttonloop++){
		if(gradelevelflags & (1<<buttonloop)){
			ReverseButton(gradeButtons[buttonloop]);
		} else {
			UnreverseButton(gradeButtons[buttonloop]);
		}
	}
}

/* parsegrades:
 *  initialize gradelevelflags, according to ascii string
 *	(containing single word)
 *	If string has no acceptable grade levels, we will
 *	set to ALL grades.
 */
void parsegrades(char *gradestring){
	char *parse = gradestring;
	gradelevelflags = 0;

	while(*parse && (!isspace(*parse )) ){
	    switch(*parse){
		case '1': case '2':
		case '3': case '4':
		case '5': case '6':
			gradelevelflags |= (1 << (*parse -  '0') );
			break;
		case '+':
			gradelevelflags |= ABOVESIXGRADE;
			break;
	    }
	    parse++;
	}

	if(gradelevelflags == 0)
		gradelevelflags = ALLGRADES;
}

/* GradeCalback:
 *	calback for different grade buttons
 */
void GradeCallback(Widget w,XtPointer data,XtPointer calldata){
	char statusbuff[100];
	int grademask;
	grademask = 1 << (int) data;
	
	switch((int) data){
		case 1:	case 2:
		case 3:	case 4:
		case 5: case 6:
		case 7: /* + */
			/* already on? */
			if(gradelevelflags & grademask){
				gradelevelflags -= grademask;
				if(! gradelevelflags) {
					setstatus("Must have at least one grade level!");
					Beep();
					gradelevelflags |= grademask;
					return;
				} else {
					CountKanji();
					if(numberofkanji < HAVE_AT_LEAST ) {
						setstatus("Too few kanji");
						gradelevelflags |= grademask;
						Beep();
						CountKanji();
						return;
						
					}
					UnreverseButton(w);
				}
                   	} else {
			/* nope.. turn on grade level */
				ReverseButton(w);
				gradelevelflags |= grademask;
			}
			break;

		case 0:/* "All" button */
			gradelevelflags = ALLGRADES;
			printgrades();
			break;
	}
	sprintf(statusbuff,"%d characters active",numberofkanji);
	setstatus(statusbuff);
}

/* MakeGradeSelect()
 *	create buttons to select grades 1-6, plus "all" and "+"  buttons
 */
void MakeGradeSelect(){
	Widget gradelevelLabel;
	
	int widgetcount;
	gradelevelForm = XtVaCreateManagedWidget(
			"gradedisplay",formWidgetClass,buttonform,
			XtNhorizDistance,GRADEDISTANCE,
			XtNfromHoriz,usefilebutton,
			XtNvertDistance,20,
			XtNresize,False,
			NULL);

	gradelevelLabel = XtVaCreateManagedWidget(
			"gradelabel",labelWidgetClass,gradelevelForm,
			XtNlabel,"        Grade Select",XtNborderWidth,0,
			NULL);

	gradeButtons[1] = XtVaCreateWidget("grade1",commandWidgetClass,gradelevelForm,
			 XtNlabel,"1",
			 XtNfromVert,gradelevelLabel,NULL);

	gradeButtons[2] = XtVaCreateWidget("grade2",commandWidgetClass,gradelevelForm,
			 XtNlabel,"2",XtNfromHoriz,gradeButtons[1],
			 XtNfromVert,gradelevelLabel,NULL);
	gradeButtons[3] = XtVaCreateWidget("grade3",commandWidgetClass,gradelevelForm,
			 XtNlabel,"3",XtNfromHoriz,gradeButtons[2],
			 XtNfromVert,gradelevelLabel,NULL);
	gradeButtons[4] = XtVaCreateWidget("grade4",commandWidgetClass,gradelevelForm,
			 XtNlabel,"4",XtNfromHoriz,gradeButtons[3],
			 XtNfromVert,gradelevelLabel,NULL);
	gradeButtons[5] = XtVaCreateWidget("grade5",commandWidgetClass,gradelevelForm,
			 XtNlabel,"5",XtNfromHoriz,gradeButtons[4],
			 XtNfromVert,gradelevelLabel,NULL);
	gradeButtons[6] = XtVaCreateWidget("grade6",commandWidgetClass,gradelevelForm,
			 XtNlabel,"6",XtNfromHoriz,gradeButtons[5],
			 XtNfromVert,gradelevelLabel,NULL);
	gradeButtons[7] = XtVaCreateWidget("grade+",commandWidgetClass,gradelevelForm,
			 XtNlabel,"+",XtNfromHoriz,gradeButtons[6],
			 XtNfromVert,gradelevelLabel,NULL);

	gradeButtons[0] = XtVaCreateWidget("allgrades",commandWidgetClass,gradelevelForm,
			  XtNlabel," All ", XtNfromHoriz,gradeButtons[7],
			  XtNfromVert,gradelevelLabel,NULL);

	XtManageChildren(gradeButtons,8);

	for(widgetcount=0;widgetcount<8;widgetcount++){
		XtAddCallback(gradeButtons[widgetcount],XtNcallback,GradeCallback,(XtPointer) widgetcount);
	}
	/* select ALL.. */
	
}
